@@ -21,6 +21,7 @@
 #define __GCAL_MANAGER_H__
 
 #include "e-cal-data-model.h"
+#include "gcal-clock.h"
 #include "gcal-event.h"
 
 #include <libical/icaltime.h>
@@ -74,10 +75,12 @@ void                 gcal_manager_create_event                   (GcalManager
                                                                   GcalEvent          *event);
 
 void                 gcal_manager_update_event                   (GcalManager        *manager,
-                                                                  GcalEvent          *event);
+                                                                  GcalEvent          *event,
+                                                                  ECalObjModType      mod);
 
 void                 gcal_manager_remove_event                   (GcalManager        *manager,
-                                                                  GcalEvent          *event);
+                                                                  GcalEvent          *event,
+                                                                  ECalObjModType      mod);
 
 void                 gcal_manager_move_event_to_source           (GcalManager        *manager,
                                                                   GcalEvent          *event,
@@ -103,6 +106,9 @@ GList*               gcal_manager_get_events                     (GcalManager
 
 gboolean             gcal_manager_get_loading                    (GcalManager        *self);
 
+/* Clock */
+GcalClock*           gcal_manager_get_clock                      (GcalManager        *self);
+
 /* Online Accounts */
 GoaClient*           gcal_manager_get_goa_client                 (GcalManager        *manager);
 
