@@ -6,7 +6,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -240,7 +240,7 @@ gcal_subscriber_view_add (GtkContainer *container,
   if (gcal_event_is_multiday (event))
     {
       priv->multi_cell_children = g_list_insert_sorted (priv->multi_cell_children, widget,
-                                                       (GCompareFunc) gcal_event_widget_compare_by_length);
+                                                       (GCompareFunc) gcal_event_widget_sort_events);
     }
   else
     {
