@@ -20,7 +20,6 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include "gcal-application.h"
-#include "gcal-log.h"
 
 #include <glib/gi18n.h>
 
@@ -32,8 +31,6 @@ main (int   argc,
   GcalApplication *app;
   int status;
 
-  gcal_log_init ();
-
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
