@@ -67,7 +67,6 @@ struct _GcalWeekHeader
   gboolean          expanded;
 
   gboolean          use_24h_format;
-  gint              redraw_timeout_id;
 
   icaltimetype     *active_date;
 
@@ -140,7 +139,7 @@ on_button_pressed (GcalWeekHeader *self,
 
   gtk_widget_queue_draw (GTK_WIDGET (self));
 
-  return GDK_EVENT_STOP;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -976,14 +975,6 @@ on_expand_action_activated (GcalWeekHeader *self,
     header_expand (self);
 }
 
-static gboolean
-on_redraw_timeout_cb (gpointer data)
-{
-  gtk_widget_queue_draw (data);
-
-  return G_SOURCE_CONTINUE;
-}
-
 /* Drawing area content and size */
 static gdouble
 get_weekday_names_height (GtkWidget *widget)
@@ -1038,12 +1029,6 @@ gcal_week_header_finalize (GObject *object)
 
   g_clear_pointer (&self->active_date, g_free);
 
-  if (self->redraw_timeout_id > 0)
-    {
-      g_source_remove (self->redraw_timeout_id);
-      self->redraw_timeout_id = 0;
-    }
-
   for (i = 0; i < 7; i++)
     g_list_free (self->events[i]);
 }
@@ -1193,7 +1178,7 @@ gcal_week_header_draw (GtkWidget      *widget,
 
   for (i = 0; i < 7; i++)
     {
-      gchar *weekday_date, *weekday_abv;
+      gchar *weekday_date, *weekday_abv, *weekday;
       gdouble x;
       gint font_width;
       gint n_day;
@@ -1232,7 +1217,9 @@ gcal_week_header_draw (GtkWidget      *widget,
       gtk_style_context_restore (context);
 
       /* Draws the days name */
-      weekday_abv = g_strdup_printf ("%s", g_utf8_strup (gcal_get_weekday ((i + self->first_weekday) % 7), -1));
+      weekday = g_utf8_strup (gcal_get_weekday ((i + self->first_weekday) % 7), -1);
+      weekday_abv = g_strdup_printf ("%s", weekday);
+      g_free (weekday);
 
       gtk_style_context_save (context);
       gtk_style_context_add_class (context, "week-names");
@@ -1290,14 +1277,6 @@ gcal_week_header_draw (GtkWidget      *widget,
 
   GTK_WIDGET_CLASS (gcal_week_header_parent_class)->draw (widget, cr);
 
-  /* Fire the redraw timeout if needed */
-  if (self->redraw_timeout_id == 0)
-    {
-      self->redraw_timeout_id = g_timeout_add_seconds (5,
-                                                       (GSourceFunc) on_redraw_timeout_cb,
-                                                       self);
-    }
-
   g_clear_pointer (&week_start, g_date_time_unref);
   g_clear_pointer (&week_end, g_date_time_unref);
 
@@ -1353,6 +1332,7 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
   g_autoptr (GDateTime) week_start;
   g_autoptr (GDateTime) dnd_date;
   g_autoptr (GDateTime) new_end;
+  g_autoptr (GDateTime) tmp_dt;
   GDateTime *start_date;
   GDateTime *end_date;
   GTimeSpan difference;
@@ -1369,7 +1349,10 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
   drop_cell = get_dnd_cell (widget, x, y);
   event_widget = gtk_drag_get_source_widget (context);
 
-  week_start = dnd_date = NULL;
+  week_start = NULL;
+  dnd_date = NULL;
+  new_end = NULL;
+  tmp_dt = NULL;
 
   if (!GCAL_IS_EVENT_WIDGET (event_widget))
     return FALSE;
@@ -1391,21 +1374,22 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
        * The only case where we don't touch the timezone is for
        * timed, multiday events.
        */
-      dnd_date = g_date_time_new (gcal_event_get_timezone (event),
-                                  g_date_time_get_year (week_start),
-                                  g_date_time_get_month (week_start),
-                                  g_date_time_get_day_of_month (week_start) + drop_cell,
-                                  g_date_time_get_hour (start_date),
-                                  g_date_time_get_minute (start_date),
-                                  0);
+      tmp_dt = g_date_time_new (gcal_event_get_timezone (event),
+                                g_date_time_get_year (week_start),
+                                g_date_time_get_month (week_start),
+                                g_date_time_get_day_of_month (week_start),
+                                g_date_time_get_hour (start_date),
+                                g_date_time_get_minute (start_date),
+                                0);
     }
   else
     {
-      dnd_date = g_date_time_new_utc (g_date_time_get_year (week_start),
-                                      g_date_time_get_month (week_start),
-                                      g_date_time_get_day_of_month (week_start) + drop_cell,
-                                      0, 0, 0);
+      tmp_dt = g_date_time_new_utc (g_date_time_get_year (week_start),
+                                    g_date_time_get_month (week_start),
+                                    g_date_time_get_day_of_month (week_start),
+                                    0, 0, 0);
     }
+  dnd_date = g_date_time_add_days (tmp_dt, drop_cell);
 
   /* End date */
   difference = turn_all_day ? 24 : g_date_time_difference (end_date, start_date) / G_TIME_SPAN_HOUR;
@@ -1423,7 +1407,7 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
     gcal_event_set_all_day (event, TRUE);
 
   /* Commit the changes */
-  gcal_manager_update_event (self->manager, event);
+  gcal_manager_update_event (self->manager, event, E_CAL_OBJ_MOD_THIS);
 
   /* Cancel the DnD */
   self->dnd_cell = -1;
@@ -1503,6 +1487,11 @@ gcal_week_header_init (GcalWeekHeader *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  /* This is to avoid stray lines when adding and removing events */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  gtk_container_set_reallocate_redraws (GTK_CONTAINER (self), TRUE);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
   /* Setup the week header as a drag n' drop destination */
   gtk_drag_dest_set (GTK_WIDGET (self),
                      0,
@@ -1519,6 +1508,11 @@ gcal_week_header_set_manager (GcalWeekHeader *self,
   g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
 
   self->manager = manager;
+
+  g_signal_connect_swapped (gcal_manager_get_clock (manager),
+                            "day-changed",
+                            G_CALLBACK (gtk_widget_queue_draw),
+                            self);
 }
 
 void
@@ -1664,13 +1658,13 @@ gcal_week_header_remove_event (GcalWeekHeader *self,
       if (event_position == -1)
         continue;
 
-      /* Move remaining events up */
-      move_events_at_column (self, UP, weekday, event_position);
-
       /* Remove from the current weekday */
       l = g_list_remove (l, removed_event);
       self->events[weekday] = l;
 
+      /* Move remaining events up */
+      move_events_at_column (self, UP, weekday, event_position);
+
     }
 
   /* Check if we eventually can merge events */
